function [p, table, stats, terms] = anovan (c, cluster, k, factors, varargin)

%CORTEX/ANOVAN  n-way analysis of variance on a @cortex object
%
%   [P,T,STATS,TERMS] = ANOVAN(CO,CLUSTER,KO,FACTORS) performs n-way
%   ANOVA on firing rates from spike CLUSTER in object CO. KO is a
%   @key object that includes predefined factors (subsets of keys
%   associated to 2 or more levels, see help key/addfactor); FACTORS
%   is a numeric array specifying the factors to be analyzed from
%   those embedded in KO. Data are analyzed over the epoch specified
%   by the keys. See Matlab's ANOVAN help text for information on the
%   output arguments.
%
%   The following additional arguments can be specified in order to
%   fine-tune the analysis:
%   [...] = ANOVAN(...,'MODEL')
%   [...] = ANOVAN(...,'MODEL',SSTYPE)
%   [...] = ANOVAN(...,'MODEL',SSTYPE,'DISPLAYOPT')
%
%   See Matlab's ANOVAN help text for an explanation of these arguments.
%   Note that unlike Matlab's ANOVAN, factors are automatically labeled
%   (i.e., the GNAME argument is not available here) and DISPLAYOPT is
%   'off' by default.
%
%Last modified: 23 Mar 06

% Since 16 Feb 05
% 23 Mar 06: fixed bug in passing level labels to addata

factors = horzarray(factors);
nfactors = length (factors);
as = facts (nfactors);
ks = struct(k);
for ke = 1:k.len
   foundlev=0;
   for f = factors
      if ~isempty (ks.f.levels{ke,f}), foundlev=foundlev+1; end
   end
   if foundlev
      if foundlev < nfactors
         disp (['unbalanced factor! ignoring key #' num2str(ke)]);
      else
         tfr = trialfiringrate (c, cluster, k(ke));
			[lev{1:nfactors}] = ks.f.levels{ke, factors};
         as = addata (as, lev, tfr);
      end
   end
end

% the default anovan argument list 
arglist = {'linear' 3 {ks.f.factors{factors}} 'off'};
% modify the list if matching arguments were passed
lv = length(varargin);
if lv>0, arglist{1} = varargin {1}; end % model
if lv>1, arglist{2} = varargin {2}; end % sstype
if lv>2, arglist{4} = varargin {3}; end % displayopt
[p, table, stats, terms] = anovan (as, arglist{:});
