function [lout, sh, fh] = raster (spmat, xlim, tsize);

%RASTER   draws a raster plot on the current axis
%
%   RASTER(SPMAT,XLIM)  creates a raster plot of a neuron's
%   firing activity, based on the SPMAT spikematrix.
%
%   XLIM specifies the time in ms corresponding to the first column in
%   SPMAT, relative to the cue event. This is needed to properly set
%   the axis' 'xlim' property. If omitted, XLIM defaults to 1.
%
%   LH = RASTER(...) also returns a vector of handles to the lines
%   representing the raster's tick marks.
%
%   If the spikematrix is computed with the include_short_trials setting
%   on, then the absence of ticks towards the beginning or the end of the
%   trial indicates lack of data collection rather than lack of cell
%   activity. Such epochs are then marked with ticks of different styles.
%   For example, a green and a red tick could mark, respectively, the
%   beginning and the end of an epoch during which spikes were actually
%   recorded.
%
%   To change the style of the tick marks edit giucode.def
%
% Last modified: 22 Jun 06

% 14 Jul 00: numerous changes, including group handling,
%            SPMAT handling, and removal of figure generation.
% 13 Oct 00: bug: xs=r(2).finish;  -->  xf=r(2).finish;
%  3 May 04: function overloaded. See also @cortex/raster.m
% 27 Oct 04: bug in argument checking code; cosmetics
% 10 Feb 06: output handles only if nargout
% 15 Feb 06: getdefs of GiuCoDefaulValues;
%            tick mark features moved to prefs
% 22 Jun 06: two small bugs fixed

error (NOTspikematrix(spmat));
ds = getdefs ('spikes');
[rows, cols] = size (spmat);

if nargin==1
   xlim = [1, cols];
else
    error (NOTscalar (xlim, 'XLIM'));
    xlim(2) = xlim(1) + cols;
    if nargin>2
        warning ('too many arguments! tick mark specs are now set in giucode.def');
    end
end

nans = isnan(spmat);
spmat (nans) = 0;

ts = ds.raster_tick_length;
[ly, lx] = find (spmat);
ly = [ly' + ts(2); ly' + ts(1)];
lx = [lx' + xlim(1); lx' + xlim(1)];
lh = line (lx, ly);
axis ij;
set (gca, 'ylim', [1 rows+1], 'xlim', xlim, 'tickdir', 'out');
set (lh, 'color', ds.raster_spike_color);

sh = []; fh = [];
% do the following only if there are short trials
if sum(sum(nans))
    sy = []; fy = [];
    sx = []; fx = [];
    for i = 1:rows
        % this will highlight changes in the sequence of 0s and 1s
        td = diff(nans (i,:));
        % looks for the end of an initial series of NaNs
        st = find (td == -1);
        if ~isempty (st)
            sy = [sy; i];
            sx = [sx; st];
        end
        % looks for the beginning of a final series of NaNs
        st = find (td == 1);
        if ~isempty (st)
            fy = [fy; i];
            fx = [fx; st+1];
        end
    end
    sy = [sy' + ts(2); sy' + ts(1)];
    sx = [sx' + xlim(1); sx' + xlim(1)];
    sh = line (sx, sy);
    set (sh, 'color', ds.raster_start_color);
    
    fy = [fy' + ts(2); fy' + ts(1)];
    fx = [fx' + xlim(1); fx' + xlim(1)];
    fh = line (fx, fy);
    set (fh, 'color', ds.raster_finish_color);
end

set ([lh; sh; fh], 'linewidth', ds.raster_tick_width)

if nargout, lout=lh; end
