function as = addata (as, levels, data)

%@FACTS/ADDATA adds observations to a @facts object
%
%   F = ADDATA(F,LEVELS,DATA) adds to F the set of observations
%   contained in the numeric vector DATA; LEVELS specifies the levels
%   associated to the data (one level per factor, obviously). LEVELS can be
%   either a numeric vector or a cell array of strings.
%
%   Note that the object variable must appear both as the first input
%   argument and as the output.
%
%Last modified: 26 Aug 06

% Changes since 10 Mar 06
% 26 Aug 06: warnings only displayed if verbose
% 27 Aug 06: multiple dependent var support

% handle data
if isnumeric(data)
   cols = size (data, 2);
   if cols ~= as.X
      error ('the number of columns in data matrix does not match the number of dependent variables in object');
   end
   as.D = [as.D; data];
elseif isstruct(data)
   fn = fieldnames(data);
   nn = length(data.(fn{1}));
   
   % create a temporary NaN matrix to be filled with data
   td = repmat (NaN, nn, as.X);
   for f=fn
      g = f{1};
      d = data.(g);
      foundep = strmatch (g, as.P);
      if isempty(foundep)
         error ([g ' is not a dependent variable in this object']);
      end
      if length(d) ~= nn
         error ('all dependent variables should have the same number of cases');
      end
      td(:,foundep) = d;
   end
   as.D = [as.D; td];
else
   error ('DATA must be either a numeric matrix or a structure');
end

% handle levels
if isnumeric (levels)
	error (NOTnumvector(levels));
	levels = num2cell (levels);
elseif isstruct (levels)
elseif ~iscell(levels)
	error ('LEVELS must be either a numeric vector or a cell array');
end
if length (levels) ~= as.N
   error  (['wrong number of levels: this is a ' num2str(as.N) '-factor @fact object']);
end

for ii = 1:as.N
	lev = levels{ii};
	ln = length (as.L{ii});
	if ischar(lev)
		m = strmatch (lev, as.L{ii});
		if isempty (m)
         if as.verbose
            disp (['adding new level "' lev '" to factor "' as.F{ii} '"']);
         end
         m = ln+1;
         as.L{ii}{m} = lev;
		end
	else
		if lev < 1
         error ('level number must be >0');
		elseif lev > ln
			m = ln + 1;
         if as.verbose
            disp(['adding level ' num2str(m) ' to factor "' as.F{ii} '"']);
         end
			as.L{ii}{m} = sprintf ('level%02d', m);
		else
			m = lev;
		end
	end
   rr = repmat (as.L{ii}(m), length(data), 1);
   as.G {ii} = [as.G{ii}; rr];
end
% if as.verbose, info (as); end
